import checkArray from './_checkArray';

const processViaPostMessage = (ar, fn, resolve, chunkSize, msg) => {
    const values = [...ar];

    window.addEventListener('message', execNext);
    postMessage(msg, '*');

    function execNext ({ data }) {
        if (data !== msg) { return; }

        if (values.length) {
            values.splice(0, chunkSize).forEach(fn);
            postMessage(msg, '*');
        } else {
            resolve();
            window.removeEventListener('message', execNext);
        }
    }
};

const processViaTimeout = (ar, fn, resolve, chunkSize, msg) => {
    if (ar.length) {
        ar.slice(0, chunkSize).forEach(fn);
        setTimeout(processViaTimeout.bind(this, ar.slice(chunkSize), fn, resolve, chunkSize, msg));
    } else {
        resolve();
    }
};

const createSplitMessage = () => `FOR_EACH_SPLIT_${Math.floor(Math.random() * 10000)}`;

const process = typeof postMessage === 'undefined'
    ? processViaPostMessage
    : processViaTimeout;

/**
 * Splits an array into chunks and breaks the execution queue to ensure reactivity.
 *
 * @memberOf array
 * @param {Array} ar
 * @param {function} fn
 * @param {number} chunkSize
 * @return {Promise}
 */
const chunkedForEach = (ar = [], fn = Function.prototype, chunkSize = 1) => {
    checkArray(ar);

    return new Promise((resolve, reject) => {
        process(ar, fn, resolve, chunkSize, createSplitMessage());
    });
};

export default chunkedForEach;
